/*****************************************************************************
 * douze.h - Midi utilities
 *****************************************************************************
 * Copyright (C) 2001 Arnaud de Bossoreille de Ribou
 *
 * Authors: Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/


/*****************************************************************************
 * Description :
 *
 *   - misc midi structures
 *   - midi demultiplexer structure and types
 *****************************************************************************/


#ifndef __INC_DOUZE_H__
#define __INC_DOUZE_H__


/*****************************************************************************
 * logging string
 *****************************************************************************/
#define DOUZE_DEMUX         "midi demux : "


/*****************************************************************************
 * douze_packet_t
 *****************************************************************************
 * midi packet structure to store midi messages' payload
 *****************************************************************************/
typedef struct douze_packet_s
{
  byte_t *                  p_buffer_start;     /* start of the buffer */
  byte_t *                  p_buffer_end;       /* end of the buffer */

  byte_t *                  p_payload_start;    /* start of the payload */
  byte_t *                  p_payload_end;      /* end of the payload */

  struct douze_packet_s *   p_next;             /* next packet */

} douze_packet_t;


/*****************************************************************************
 * douze_message_t
 *****************************************************************************
 * midi message structure which contains midi packets
 *****************************************************************************/
typedef struct douze_message_s
{
  unsigned int          ui_message_size;        /* size in bytes */

  douze_packet_t *      p_first;                /* first packet */
  douze_packet_t *      p_last;                 /* last packet */

  douze_packet_t *      p_current;              /* current packet */
  byte_t *              p_byte;                 /* current byte */

} douze_message_t;


/*****************************************************************************
 * usefull callback function types
 *****************************************************************************/
typedef douze_packet_t *        (* douze_GetPacketFunc)(void * p_data);
typedef void                    (* douze_ReleasePacketFunc)(void * p_data,
                                                douze_packet_t * p_packet);


/*****************************************************************************
 * douze_demux_t
 *****************************************************************************
 * midi demultiplexer structure
 *****************************************************************************/
typedef struct douze_demux_s
{
  /* callback functions used by the demultiplexer to
     - get a packet;
     - release a packet;
     - parse a valid message.
   */
  douze_GetPacketFunc           pf_get_packet;
  douze_ReleasePacketFunc       pf_release_packet;
  void                          (* pf_parse_message)(void * p_data,
                                                struct douze_demux_s * p_demux,
                                                douze_message_t * p_message);

  /* pointers given to each callback function */
  void *                        p_packet_cb_data;
  void *                        p_parser_cb_data;

  /* internal data */
  douze_message_t *             p_current_message;
  douze_packet_t *              p_current_packet;
  unsigned int                  b_status_expected;
  s8                            i_byte_to_read;
  byte_t                        ui_end_status;

} douze_demux_t;


/*****************************************************************************
 * usefull callback function types
 *****************************************************************************/
typedef void                    (* douze_ParseMessageFunc)(void * p_data,
                                                douze_demux_t * p_demux,
                                                douze_message_t * p_message);


/*****************************************************************************
 * douze_sysex_message_t
 *****************************************************************************
 * midi message structure which contains a midi System Exclusive message
 *****************************************************************************/
typedef struct douze_sysex_message_s
{
  unsigned int          ui_payload_size;        /* payload size in bytes */

  douze_message_t *     p_message;              /* midi message that
                                                   contain the payload */
  douze_demux_t *       p_demux;                /* midi demultiplexer to
                                                   release the message */

} douze_sysex_message_t;


/*****************************************************************************
 * douze_InitDemux
 *****************************************************************************
 * This function initializes a douze_demux_t structure.
 *****************************************************************************/
void douze_InitDemux(douze_demux_t *            p_demux,
                     douze_GetPacketFunc        pf_get_packet,
                     douze_ReleasePacketFunc    pf_release_packet,
                     douze_ParseMessageFunc     pf_parse_message,
                     void *                     p_packet_cb_data,
                     void *                     p_parser_cb_data);


/*****************************************************************************
 * douze_ReleaseDemux
 *****************************************************************************
 * This funtion releases a midi demultiplexer.
 *****************************************************************************/
void douze_ReleaseDemux(douze_demux_t * p_demux);


/*****************************************************************************
 * douze_DemuxByte
 *****************************************************************************
 * This function is used to demultiplex midi messages and calls
 * p_data_get_parse_message on valid messages (returns -1 on error).
 *****************************************************************************/
int douze_DemuxByte(douze_demux_t * p_demux, byte_t data);


/*****************************************************************************
 * douze_ReleaseMessage
 *****************************************************************************
 * This funtion releases a midi message and its packets.
 *****************************************************************************/
void douze_ReleaseMessage(douze_demux_t * p_demux,
                          douze_message_t * p_message);


/*****************************************************************************
 * douze_ReleaseSysExMessage
 *****************************************************************************
 * This funtion releases a midi SysEx message.
 *****************************************************************************/
void douze_ReleaseSysExMessage(douze_sysex_message_t * p_sysex);


/*****************************************************************************
 * douze_GetByte
 *****************************************************************************
 * Get the next byte of the message.
 *****************************************************************************/
static __inline__ u8 douze_GetByte(douze_message_t * p_message)
{
  if(p_message->p_current != NULL)
  {
    while(p_message->p_byte >= p_message->p_current->p_payload_end)
    {
      p_message->p_current = p_message->p_current->p_next;
      if(p_message->p_current != NULL)
        p_message->p_byte = p_message->p_current->p_payload_start;
      else
        break;
    }

    if(    (p_message->p_current != NULL)
        && (p_message->p_byte < p_message->p_current->p_payload_end))
      return *((p_message->p_byte)++);
    else
      /* error, should not happen */
      return 0;
  }
  else
    /* error, should not happen */
    return 0;
}


#else
#error "multiple inclusion of douze.h"
#endif

