/*****************************************************************************
 * douze_parser.h - Midi parser
 *****************************************************************************
 * Copyright (C) 2001 Arnaud de Bossoreille de Ribou
 *
 * Authors: Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/


/*****************************************************************************
 * Description :
 *
 *   - midi parser structure and types
 *****************************************************************************/


#ifndef __INC_DOUZE_PARSER_H__
#define __INC_DOUZE_PARSER_H__


/*****************************************************************************
 * logging string
 *****************************************************************************/
#define DOUZE_PARSER            "midi parser : "


/*****************************************************************************
 * usefull callback function types
 *****************************************************************************
 * The douze_message_t structure is released after each call to these
 * callback functions except for douze_OnSystemExclusiveFunc because of the
 * payload contained in it.
 *****************************************************************************/
typedef void    (* douze_OnNoteOffFunc)(void * p_data,
                                        u8 ui_channel,
                                        u8 ui_note,
                                        u8 ui_velocity);
typedef void    (* douze_OnNoteOnFunc) (void * p_data,
                                        u8 ui_channel,
                                        u8 ui_note,
                                        u8 ui_velocity);
typedef void    (* douze_OnAftertouchFunc)(void * p_data,
                                           u8 ui_channel,
                                           u8 ui_note,
                                           u8 ui_pressure);
typedef void    (* douze_OnControllerFunc)(void * p_data,
                                           u8 ui_channel,
                                           u8 ui_controller,
                                           u8 ui_value);
typedef void    (* douze_OnProgramChangeFunc)(void * p_data,
                                              u8 ui_channel,
                                              u8 ui_program);
typedef void    (* douze_OnChannelPressureFunc)(void * p_data,
                                                u8 ui_channel,
                                                u8 ui_pressure);
typedef void    (* douze_OnPitchWheelFunc)(void * p_data,
                                           u8 ui_channel,
                                           u16 ui_value);
typedef void    (* douze_OnSystemExclusiveFunc)(void * p_data,
                                            u16 ui_manufacturer_id,
                                            douze_sysex_message_t * p_sysex);
typedef void    (* douze_OnMTCQuarterFrameMsgFunc) (void * p_data,
                                                    u8 ui_time_code);
typedef void    (* douze_OnSongPositionPointerFunc)(void * p_data,
                                                    u16 ui_beat);
typedef void    (* douze_OnSongSelectFunc)(void * p_data,
                                           u8 ui_song);
/* this feature is unofficial ?! */
typedef void    (* douze_OnBusSelectFunc)(void * p_data,
                                          u8 ui_bus);
typedef void    (* douze_OnTuneRequestFunc)(void * p_data);
typedef void    (* douze_OnMidiClockFunc)(void * p_data);
typedef void    (* douze_OnTickFunc)(void * p_data);
typedef void    (* douze_OnMidiStartFunc)(void * p_data);
typedef void    (* douze_OnMidiContinueFunc)(void * p_data);
typedef void    (* douze_OnMidiStopFunc)(void * p_data);
typedef void    (* douze_OnActiveSenseFunc)(void * p_data);
typedef void    (* douze_OnResetFunc)(void * p_data);


/*****************************************************************************
 * douze_parser_t
 *****************************************************************************
 * midi parser structure
 *****************************************************************************/
typedef struct douze_parser_s
{
  /* callback functions used by the parser */
  douze_OnNoteOffFunc                   pf_on_note_off;
  douze_OnNoteOnFunc                    pf_on_note_on;
  douze_OnAftertouchFunc                pf_on_aftertouch;
  douze_OnControllerFunc                pf_on_controller;
  douze_OnProgramChangeFunc             pf_on_program_change;
  douze_OnChannelPressureFunc           pf_on_channel_pressure;
  douze_OnPitchWheelFunc                pf_on_pitch_wheel;
  douze_OnSystemExclusiveFunc           pf_on_system_exclusive;
  douze_OnMTCQuarterFrameMsgFunc        pf_on_mtc_quarter_frame_msg;
  douze_OnSongPositionPointerFunc       pf_on_song_position_pointer;
  douze_OnSongSelectFunc                pf_on_song_select;
  douze_OnBusSelectFunc                 pf_on_bus_select;
  douze_OnTuneRequestFunc               pf_on_tune_request;
  douze_OnMidiClockFunc                 pf_on_midi_clock;
  douze_OnTickFunc                      pf_on_tick;
  douze_OnMidiStartFunc                 pf_on_midi_start;
  douze_OnMidiContinueFunc              pf_on_midi_continue;
  douze_OnMidiStopFunc                  pf_on_midi_stop;
  douze_OnActiveSenseFunc               pf_on_active_sense;
  douze_OnResetFunc                     pf_on_reset;

  /* pointer given to each callback function */
  void *                                p_cb_data;

} douze_parser_t;


/*****************************************************************************
 * douze_InitDemux
 *****************************************************************************
 * This function initializes a douze_demux_t structure.
 *****************************************************************************/
void douze_InitParser(douze_parser_t *          p_parser,
                douze_OnNoteOffFunc             pf_on_note_off,
                douze_OnNoteOnFunc              pf_on_note_on,
                douze_OnAftertouchFunc          pf_on_aftertouch,
                douze_OnControllerFunc          pf_on_controller,
                douze_OnProgramChangeFunc       pf_on_program_change,
                douze_OnChannelPressureFunc     pf_on_channel_pressure,
                douze_OnPitchWheelFunc          pf_on_pitch_wheel,
                douze_OnSystemExclusiveFunc     pf_on_system_exclusive,
                douze_OnMTCQuarterFrameMsgFunc  pf_on_mtc_quarter_frame_msg,
                douze_OnSongPositionPointerFunc pf_on_song_position_pointer,
                douze_OnSongSelectFunc          pf_on_song_select,
                douze_OnBusSelectFunc           pf_on_bus_select,
                douze_OnTuneRequestFunc         pf_on_tune_request,
                douze_OnMidiClockFunc           pf_on_midi_clock,
                douze_OnTickFunc                pf_on_tick,
                douze_OnMidiStartFunc           pf_on_midi_start,
                douze_OnMidiContinueFunc        pf_on_midi_continue,
                douze_OnMidiStopFunc            pf_on_midi_stop,
                douze_OnActiveSenseFunc         pf_on_active_sense,
                douze_OnResetFunc               pf_on_reset,
                void *                          p_cb_data);


/*****************************************************************************
 * douze_ParseMessage
 *****************************************************************************
 * This function is used to parse midi messages and calls
 * the appropriate callback functions.
 *****************************************************************************/
void douze_ParseMessage(douze_parser_t * p_parser,
                        douze_demux_t * p_demux,
                        douze_message_t * p_message);


#else
#error "multiple inclusion of douze_parser.h"
#endif

