/*****************************************************************************
 * livedrive_decoder.h - Live! Drive IR Remote control decoder
 *****************************************************************************
 * Copyright (C) 2001 Arnaud de Bossoreille de Ribou
 *
 * Authors: Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/


/*****************************************************************************
 * Description :
 *
 *   - live drive decoder structures, types and data.
 *****************************************************************************/


#ifndef __INC_LIVEDRIVE_DECODER_H__
#define __INC_LIVEDRIVE_DECODER_H__


/*****************************************************************************
 * logging string
 *****************************************************************************/
#define LIVEDRIVE_DECODER      "live drive decoder : "  


/*****************************************************************************
 * creative_dev_desc_t
 *****************************************************************************
 * Creative device description structure
 *****************************************************************************/
typedef struct creative_dev_desc_s
{
  u32                   ui_dev_id;      /* device id (see below) */
  char                  psz_desc[64];   /* string description */

} creative_dev_desc_t;


/*****************************************************************************
 * creative_key_desc_t
 *****************************************************************************
 * Creative key description structure
 *****************************************************************************/
typedef struct creative_key_desc_s
{
  u32                   ui_key_id;      /* key id (see below) */
  u32                   ui_dev_id;      /* device id (see below) */
  char                  psz_desc[64];   /* string description */

} creative_key_desc_t;


/*****************************************************************************
 * Device IDs (arbitrary)
 *****************************************************************************/
#define CL_DEVID_NULL                   0

#define CL_DEVID_RM900                  1
#define CL_DEVID_DTT3500                2


/*****************************************************************************
 * Key IDs
 *****************************************************************************/
#define CL_KEYID_NULL                   0

/* Live! Drive IR RM-900 (Creative) */
#define CL_KEYID_RM900_1                0x2ed144c1
#define CL_KEYID_RM900_2                0x0ef144c1
#define CL_KEYID_RM900_3                0xf60944c1
#define CL_KEYID_RM900_4                0xae5144c1
#define CL_KEYID_RM900_5                0xde2144c1
#define CL_KEYID_RM900_6                0xe11e44c1
#define CL_KEYID_RM900_7                0x6e9144c1
#define CL_KEYID_RM900_8                0x3ec144c1
#define CL_KEYID_RM900_9                0x11ee44c1
#define CL_KEYID_RM900_0                0xfe0144c1

#define CL_KEYID_RM900_STOP             0x5ea144c1
#define CL_KEYID_RM900_PLAYPAUSE        0x619e44c1
#define CL_KEYID_RM900_PREV             0x01fe44c1
#define CL_KEYID_RM900_NEXT             0xa15e44c1
#define CL_KEYID_RM900_SLOW             0x41be44c1
#define CL_KEYID_RM900_STEP             0x817e44c1

#define CL_KEYID_RM900_MUTE             0x897644c1
#define CL_KEYID_RM900_VOLMINUS         0x39c644c1
#define CL_KEYID_RM900_VOLPLUS          0xb94644c1

#define CL_KEYID_RM900_EAX              0xce3144c1
#define CL_KEYID_RM900_OPTIONS          0xbe4144c1
#define CL_KEYID_RM900_DISPLAY          0x916e44c1
#define CL_KEYID_RM900_RETURN           0x8e7144c1
#define CL_KEYID_RM900_START            0xee1144c1
#define CL_KEYID_RM900_CLOSE            0xc13e44c1

#define CL_KEYID_RM900_UP               0x21de44c1
#define CL_KEYID_RM900_DOWN             0x4eb144c1
#define CL_KEYID_RM900_LEFT             0x1ee144c1
#define CL_KEYID_RM900_RIGHT            0x51ae44c1
#define CL_KEYID_RM900_SELECT           0x7e8144c1

/* Cambridge SoundWorks DTT3500 (Creative) */
#define CL_KEYID_DTT3500_POWER          0x788744c1

#define CL_KEYID_DTT3500_VOLPLUS        0xb84744c1
#define CL_KEYID_DTT3500_VOLMINUS       0x38c744c1

#define CL_KEYID_DTT3500_MUTE           0x887744c1
#define CL_KEYID_DTT3500_TEST           0x08f744c1

#define CL_KEYID_DTT3500_CMSS           0x708f44c1
#define CL_KEYID_DTT3500_MULTICHANNEL   0x28d744c1
#define CL_KEYID_DTT3500_PROLOGIC       0x58a744c1
#define CL_KEYID_DTT3500_DYNAMICMODE    0xb04f44c1
#define CL_KEYID_DTT3500_DOLBYDIGITAL   0xc83744c1
#define CL_KEYID_DTT3500_AUDIOMODE      0x986744c1


/*****************************************************************************
 * usefull callback function types
 *****************************************************************************/
typedef void    (* livedrive_HandleKeyFunc)(void * p_data, u32 ui_key_id);


/*****************************************************************************
 * livedrive_decoder_t
 *****************************************************************************
 * live drive decoder structure
 *****************************************************************************/
typedef struct livedrive_decoder_s
{
  /* callback function */
  livedrive_HandleKeyFunc       pf_handle_key;
  /* callback data */
  void *                        p_cb_data;

} livedrive_decoder_t;


/*****************************************************************************
 * livedrive_InitDecoder
 *****************************************************************************
 * This function initializes a livedrive_decoder_t structure.
 *****************************************************************************/
void livedrive_InitDecoder(livedrive_decoder_t *        p_decoder,
                           livedrive_HandleKeyFunc      pf_handle_key,
                           void *                       p_cb_data);


/*****************************************************************************
 * parser callback functions
 *****************************************************************************/
void livedrive_OnSystemExclusive(livedrive_decoder_t * p_decoder,
                                 u16 ui_manufacturer_id,
                                 douze_sysex_message_t * p_sysex);


/*****************************************************************************
 * creative_FindDevDesc
 *****************************************************************************
 * Find the structure associated to a device id.
 *****************************************************************************/
creative_dev_desc_t * creative_FindDevDesc(u32 ui_dev_id);


/*****************************************************************************
 * creative_FindKeyDesc
 *****************************************************************************
 * Find the structure associated to a key id.
 *****************************************************************************/
creative_key_desc_t * creative_FindKeyDesc(u32 ui_key_id);


#else
#error "multiple inclusion of livedrive_decoder.h"
#endif

