/*****************************************************************************
 * douze_demux.c - Midi demultiplexer
 *****************************************************************************
 * Copyright (C) 2001 Arnaud de Bossoreille de Ribou
 *
 * Authors: Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/


/*****************************************************************************
 * Description :
 *
 * the multiplexer gathers midi data given by the app into midi messages and
 * sends them back.
 *****************************************************************************/

#include <stdio.h>

#include "common.h"
#include "douze.h"


/*****************************************************************************
 * s_douze_message_len
 *****************************************************************************
 * Data bytes expected after a status byte to complete a midi message
 * (status & 0x7f).
 *****************************************************************************/
static s8 s_douze_message_len[128] =
{
     2,  2,  2 , 2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2, /* 0x8_ */
     2,  2,  2 , 2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2, /* 0x9_ */
     2,  2,  2 , 2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2, /* 0xa_ */
     2,  2,  2 , 2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2, /* 0xb_ */
     1,  1,  1 , 1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1, /* 0xc_ */
     1,  1,  1 , 1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1, /* 0xd_ */
     2,  2,  2 , 2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2, /* 0xe_ */
    -1,  1,  2 , 1, -2,  1,  0,  0,  0,  0,  0,  0,  0, -2,  0,  0  /* 0xf_ */
};


/*****************************************************************************
 * douze_InitDemux
 *****************************************************************************
 * This function initializes a douze_demux_t structure.
 *****************************************************************************/
void douze_InitDemux(douze_demux_t *            p_demux,
                     douze_GetPacketFunc        pf_get_packet,
                     douze_ReleasePacketFunc    pf_release_packet,
                     douze_ParseMessageFunc     pf_parse_message,
                     void *                     p_packet_cb_data,
                     void *                     p_parser_cb_data)
{
  p_demux->pf_get_packet                = pf_get_packet;
  p_demux->pf_release_packet            = pf_release_packet;
  p_demux->pf_parse_message             = pf_parse_message;
  p_demux->p_packet_cb_data             = p_packet_cb_data;
  p_demux->p_parser_cb_data             = p_parser_cb_data;

  p_demux->p_current_message = NULL;
}


/*****************************************************************************
 * douze_ReleaseDemux
 *****************************************************************************
 * This funtion releases a midi demultiplexer.
 *****************************************************************************/
void douze_ReleaseDemux(douze_demux_t * p_demux)
{
  if(p_demux->p_current_message)
    douze_ReleaseMessage(p_demux, p_demux->p_current_message);
}


/*****************************************************************************
 * douze_InitMessage
 *****************************************************************************
 * This function initializes a douze_message_t structure and its first
 * packet (returns -1 on error).
 *****************************************************************************/
static __inline__ int douze_InitMessage(douze_demux_t * p_demux,
                                        douze_message_t * p_message)
{
  douze_packet_t * p_new = p_demux->pf_get_packet(p_demux->p_packet_cb_data);

  if(p_new != NULL)
  {
    p_message->p_first = p_message->p_last = p_message->p_current = p_new;
    p_message->p_byte = p_new->p_payload_start =
                        p_new->p_payload_end = p_new->p_buffer_start;
    p_new->p_next = NULL;
    p_message->ui_message_size = 0;
    return 0;
  }

  return -1;
}


/*****************************************************************************
 * douze_ReInitMessage
 *****************************************************************************
 * This function reinitializes the midi message (release of all the packets
 * but the first one (no errors).
 *****************************************************************************/
static __inline__ void douze_ReInitMessage(douze_demux_t * p_demux,
                                           douze_message_t * p_message)
{
  douze_packet_t * p_packet = p_message->p_first->p_next;

  while(p_packet != NULL)
  {
    douze_packet_t * p_next = p_packet->p_next;
    p_demux->pf_release_packet(p_demux->p_packet_cb_data, p_packet);
    p_packet = p_next;
  }

  p_message->p_byte = p_message->p_first->p_payload_start =
      p_message->p_first->p_payload_end = p_message->p_first->p_buffer_start;
  p_message->p_last = p_message->p_first;
  p_message->p_first->p_next = NULL;
  p_message->ui_message_size = 0;
}


/*****************************************************************************
 * douze_AppendNewPacket
 *****************************************************************************
 * This function appends a new packet to the midi message (returns NULL on
 * error).
 *****************************************************************************/
static __inline douze_packet_t *
                        douze_AppendNewPacket(douze_demux_t * p_demux,
                                              douze_message_t * p_message)
{
  douze_packet_t * p_new = p_demux->pf_get_packet(p_demux->p_packet_cb_data);

  if(p_new != NULL)
  {
    p_new->p_payload_start = p_new->p_payload_end = p_new->p_buffer_start;
    p_new->p_next = NULL;
    p_message->p_last = p_message->p_last->p_next = p_new;
  }

  return p_new;
}


/*****************************************************************************
 * douze_DemuxByte
 *****************************************************************************
 * This function is used to demultiplex midi messages and calls
 * pf_parse_message on valid messages (returns -1 on error).
 *****************************************************************************/
int douze_DemuxByte(douze_demux_t * p_demux, byte_t data)
{
  boolean_t b_valid = 0;

  /* Allocate and initialize a new message structure if needed */
  if(p_demux->p_current_message == NULL)
  {
    p_demux->p_current_message = (douze_message_t *)
                                        malloc(sizeof(douze_message_t));
    if(p_demux->p_current_message == NULL)
    {
      fprintf(stderr, DOUZE_DEMUX "out of memory !!!\n");
      return -1;
    }

    /* Init the structure */
    if(douze_InitMessage(p_demux, p_demux->p_current_message))
    {
      fprintf(stderr, DOUZE_DEMUX "unable to initialize message\n");
      douze_ReleaseMessage(p_demux, p_demux->p_current_message);
      return -1;
    }

    p_demux->p_current_packet = p_demux->p_current_message->p_last;
    p_demux->b_status_expected = 1;
  }

 /* Append a packet if needed */
  if(p_demux->p_current_packet->p_payload_end >=
                                p_demux->p_current_packet->p_buffer_end)
  {
    p_demux->p_current_packet = douze_AppendNewPacket(p_demux,
                                                p_demux->p_current_message);
    if(p_demux->p_current_packet == NULL)
    {
      fprintf(stderr, DOUZE_DEMUX "netlist is empty, message trashed\n");
      douze_ReleaseMessage(p_demux, p_demux->p_current_message);
      p_demux->p_current_message = NULL;
      return -1;
    }
  }

  /* Append data */
  *(p_demux->p_current_packet->p_payload_end++) = data;
  p_demux->p_current_message->ui_message_size++;
  
/* usefull local macro */
#ifdef douze_reinit
#error "(name conflict) reinit already defined"
#endif

#define douze_reinit()                                                        \
  {                                                                           \
    fprintf(stderr, DOUZE_DEMUX "received an unexpected status\n");           \
    douze_ReInitMessage(p_demux, p_demux->p_current_message);                 \
    p_demux->p_current_packet = p_demux->p_current_message->p_last;           \
    if(data != 0xf7)                                                          \
    {                                                                         \
      *(p_demux->p_current_packet->p_payload_end++) = data;                   \
      p_demux->i_byte_to_read = s_douze_message_len[data & 0x7f];             \
      if(p_demux->i_byte_to_read == 0)                                        \
        b_valid = 1;                                                          \
      else if(p_demux->i_byte_to_read == -1)                                  \
      {                                                                       \
        if(data == 0xf0)                                                      \
          p_demux->ui_end_status = 0xf7;                                      \
      }                                                                       \
      else if(p_demux->i_byte_to_read == -2)                                  \
      {                                                                       \
        fprintf(stderr, DOUZE_DEMUX                                           \
                        "mailto:bozo@via.ecp.fr : error 2\n");                \
        douze_ReleaseMessage(p_demux, p_demux->p_current_message);            \
        return -1;                                                            \
      }                                                                       \
    }                                                                         \
    else                                                                      \
    {                                                                         \
      p_demux->b_status_expected = 1;                                         \
    }                                                                         \
  }

  /* handle the byte */ 
  if(data & 0x80)
  {
    /* status */
    if(p_demux->b_status_expected)
    {
      p_demux->b_status_expected = 0;
      p_demux->i_byte_to_read = s_douze_message_len[data & 0x7f];
      if(p_demux->i_byte_to_read == 0)
      {
        if(data != 0xf7)
          b_valid = 1;
        else
          p_demux->p_current_packet->p_payload_end--;
      }
      else if(p_demux->i_byte_to_read == -1)
      {
        if(data == 0xf0)
          p_demux->ui_end_status = 0xf7;
      }
      else if(p_demux->i_byte_to_read == -2)
      {
        fprintf(stderr, DOUZE_DEMUX "mailto:bozo@via.ecp.fr : error 1\n");
        douze_ReleaseMessage(p_demux, p_demux->p_current_message);
        return -1;
      }
    }
    else if(p_demux->i_byte_to_read == -1)
    {
      if(data == p_demux->ui_end_status)
        b_valid = 1;
      else
        douze_reinit(); /* macro */
    }
    else
    {
      douze_reinit(); /* macro */
    }
  }
  else
  {
    /* data */
    if(!p_demux->b_status_expected)
    {
      if(p_demux->i_byte_to_read > 1)
        p_demux->i_byte_to_read--;
      else if(p_demux->i_byte_to_read > 0)
        b_valid = 1;
    }
    else
    {
      fprintf(stderr, DOUZE_DEMUX
              "received data wheras status expected : 0x%02x (trashed)\n",
              data);
      p_demux->p_current_packet->p_payload_end--;
    }
  }

#undef reinit

  if(b_valid)
  {
    p_demux->pf_parse_message(p_demux->p_parser_cb_data,
                              p_demux, p_demux->p_current_message);
    p_demux->p_current_message = NULL;
  }

  return 0;
}


/*****************************************************************************
 * douze_ReleaseMessage
 *****************************************************************************
 * This funtion releases a midi message and its packets.
 *****************************************************************************/
void douze_ReleaseMessage(douze_demux_t * p_demux,
                          douze_message_t * p_message)
{
  douze_packet_t * p_packet = p_message->p_first;
  while(p_packet != NULL)
  {
    douze_packet_t * p_next = p_packet->p_next;
    p_demux->pf_release_packet(p_demux->p_packet_cb_data, p_packet);
    p_packet = p_next;
  }
  free(p_message);
}


/*****************************************************************************
 * douze_ReleaseSysExMessage
 *****************************************************************************
 * This funtion releases a midi SysEx message.
 *****************************************************************************/
void douze_ReleaseSysExMessage(douze_sysex_message_t * p_sysex)
{
  douze_ReleaseMessage(p_sysex->p_demux, p_sysex->p_message);
  free(p_sysex);
}

