/*****************************************************************************
 * douze_parser.c - Midi parser
 *****************************************************************************
 * Copyright (C) 2001 Arnaud de Bossoreille de Ribou
 *
 * Authors: Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/


/*****************************************************************************
 * Description :
 *
 * the parser decodes midi messages sent by the demultiplexer and calls the
 * appropriate calback functions.
 *****************************************************************************/

#include <stdio.h>

#include "common.h"
#include "douze.h"
#include "douze_parser.h"


/*****************************************************************************
 * douze_InitParser
 *****************************************************************************
 * This function initializes a douze_parser_t structure.
 *****************************************************************************/
void douze_InitParser(douze_parser_t *          p_parser,
                douze_OnNoteOffFunc             pf_on_note_off,
                douze_OnNoteOnFunc              pf_on_note_on,
                douze_OnAftertouchFunc          pf_on_aftertouch,
                douze_OnControllerFunc          pf_on_controller,
                douze_OnProgramChangeFunc       pf_on_program_change,
                douze_OnChannelPressureFunc     pf_on_channel_pressure,
                douze_OnPitchWheelFunc          pf_on_pitch_wheel,
                douze_OnSystemExclusiveFunc     pf_on_system_exclusive,
                douze_OnMTCQuarterFrameMsgFunc  pf_on_mtc_quarter_frame_msg,
                douze_OnSongPositionPointerFunc pf_on_song_position_pointer,
                douze_OnSongSelectFunc          pf_on_song_select,
                douze_OnBusSelectFunc           pf_on_bus_select,
                douze_OnTuneRequestFunc         pf_on_tune_request,
                douze_OnMidiClockFunc           pf_on_midi_clock,
                douze_OnTickFunc                pf_on_tick,
                douze_OnMidiStartFunc           pf_on_midi_start,
                douze_OnMidiContinueFunc        pf_on_midi_continue,
                douze_OnMidiStopFunc            pf_on_midi_stop,
                douze_OnActiveSenseFunc         pf_on_active_sense,
                douze_OnResetFunc               pf_on_reset,
                void *                          p_cb_data)
{
  p_parser->pf_on_note_off                      = pf_on_note_off;
  p_parser->pf_on_note_on                       = pf_on_note_on;
  p_parser->pf_on_aftertouch                    = pf_on_aftertouch;
  p_parser->pf_on_controller                    = pf_on_controller;
  p_parser->pf_on_program_change                = pf_on_program_change;
  p_parser->pf_on_channel_pressure              = pf_on_channel_pressure;
  p_parser->pf_on_pitch_wheel                   = pf_on_pitch_wheel;
  p_parser->pf_on_system_exclusive              = pf_on_system_exclusive;
  p_parser->pf_on_mtc_quarter_frame_msg         = pf_on_mtc_quarter_frame_msg;
  p_parser->pf_on_song_position_pointer         = pf_on_song_position_pointer;
  p_parser->pf_on_song_select                   = pf_on_song_select;
  p_parser->pf_on_bus_select                    = pf_on_bus_select;
  p_parser->pf_on_tune_request                  = pf_on_tune_request;
  p_parser->pf_on_midi_clock                    = pf_on_midi_clock;
  p_parser->pf_on_tick                          = pf_on_tick;
  p_parser->pf_on_midi_start                    = pf_on_midi_start;
  p_parser->pf_on_midi_continue                 = pf_on_midi_continue;
  p_parser->pf_on_midi_stop                     = pf_on_midi_stop;
  p_parser->pf_on_active_sense                  = pf_on_active_sense;
  p_parser->pf_on_reset                         = pf_on_reset;
  p_parser->p_cb_data                           = p_cb_data;
}


/*****************************************************************************
 * douze_DemuxByte
 *****************************************************************************
 * This function is used to demultiplex midi messages and calls
 * pf_parse_message on valid messages (returns -1 on error).
 *****************************************************************************/
void douze_ParseMessage(douze_parser_t * p_parser,
                        douze_demux_t * p_demux,
                        douze_message_t * p_message)
{
  byte_t status = douze_GetByte(p_message);
  byte_t data1, data2;
  switch(status & 0xf0)
  {
  case 0x80:
    /* Note Off */
    if(p_parser->pf_on_note_off != NULL)
    {
      data1 = douze_GetByte(p_message);
      data2 = douze_GetByte(p_message);
      p_parser->pf_on_note_off(p_parser->p_cb_data, 
                               status & 0x0f, data1, data2);
    }
    douze_ReleaseMessage(p_demux, p_message);
    break;

  case 0x90:
    /* Note On */
    if(p_parser->pf_on_note_on != NULL)
    {
      data1 = douze_GetByte(p_message);
      data2 = douze_GetByte(p_message);
      p_parser->pf_on_note_on(p_parser->p_cb_data,
                              status & 0x0f, data1, data2);
    }
    douze_ReleaseMessage(p_demux, p_message);
    break;

  case 0xa0:
    /* Aftertouch */
    if(p_parser->pf_on_aftertouch != NULL)
    {
      data1 = douze_GetByte(p_message);
      data2 = douze_GetByte(p_message);
      p_parser->pf_on_aftertouch(p_parser->p_cb_data,
                                 status & 0x0f, data1, data2);
    }
    douze_ReleaseMessage(p_demux, p_message);
    break;

  case 0xb0:
    /* Controller */
    if(p_parser->pf_on_controller != NULL)
    {
      data1 = douze_GetByte(p_message);
      data2 = douze_GetByte(p_message);
      p_parser->pf_on_controller(p_parser->p_cb_data,
                                 status & 0x0f, data1, data2);
    }
    douze_ReleaseMessage(p_demux, p_message);
    break;

  case 0xc0:
    /* Program Change */
    if(p_parser->pf_on_program_change != NULL)
    {
      data1 = douze_GetByte(p_message);
      p_parser->pf_on_program_change(p_parser->p_cb_data,
                                     status & 0x0f, data1);
    }
    douze_ReleaseMessage(p_demux, p_message);
    break;

  case 0xd0:
    /* Channel Pressure */
    if(p_parser->pf_on_channel_pressure != NULL)
    {
      data1 = douze_GetByte(p_message);
      p_parser->pf_on_channel_pressure(p_parser->p_cb_data,
                                       status & 0x0f, data1);
    }
    douze_ReleaseMessage(p_demux, p_message);
    break;

  case 0xe0:
    /* Pitch Wheel */
    if(p_parser->pf_on_pitch_wheel != NULL)
    {
      data1 = douze_GetByte(p_message);
      data2 = douze_GetByte(p_message);
      p_parser->pf_on_pitch_wheel(p_parser->p_cb_data,
                                  status & 0x0f, ((u16)data2 << 7) | data1);
    }
    douze_ReleaseMessage(p_demux, p_message);
    break;

  case 0xf0:
    /* System */
    switch(status)
    {
    case 0xf0:
      /* System Exclusive */
      if(p_parser->pf_on_system_exclusive != NULL)
      {
        douze_sysex_message_t * p_sysex =
                (douze_sysex_message_t *)malloc(sizeof(douze_sysex_message_t));
        if(p_sysex == NULL)
        {
          fprintf(stderr, DOUZE_PARSER "out of memory !!!\n");
          douze_ReleaseMessage(p_demux, p_message);
          break;
        }

        p_message->p_last->p_payload_end--; /* remove the SysEx end code */
        p_sysex->p_message = p_message;
        p_sysex->p_demux = p_demux;

        data1 = douze_GetByte(p_message);
        if(data1)
        {
          /* classic manufacturer id */
          p_sysex->ui_payload_size = p_message->ui_message_size - 3;
          p_parser->pf_on_system_exclusive(p_parser->p_cb_data,
                                           data1, p_sysex);
        }
        else
        {
          /* extended manufacturer id */
          data1 = 0x80 | douze_GetByte(p_message);
          data2 = douze_GetByte(p_message);
          p_sysex->ui_payload_size = p_message->ui_message_size - 5;
          p_parser->pf_on_system_exclusive(p_parser->p_cb_data,
                                           ((u16)data1 << 8) | data2, p_sysex);
        }
      }
      else
      {
        douze_ReleaseMessage(p_demux, p_message);
      }
      break;

    case 0xf1:
      /* MTC Quarter Frame Message */
      if(p_parser->pf_on_mtc_quarter_frame_msg != NULL)
      {
        p_parser->pf_on_mtc_quarter_frame_msg(p_parser->p_cb_data,
                                              douze_GetByte(p_message));
      }
      douze_ReleaseMessage(p_demux, p_message);
      break;
    case 0xf2:
      /* Song Position Pointer */
      if(p_parser->pf_on_song_position_pointer != NULL)
      {
        data1 = douze_GetByte(p_message);
        data2 = douze_GetByte(p_message);
        p_parser->pf_on_song_position_pointer(p_parser->p_cb_data,
                                              ((u16)data2 << 7) | data1);
      }
      douze_ReleaseMessage(p_demux, p_message);
      break;

    case 0xf3:
      /* Song Select */
      if(p_parser->pf_on_song_select != NULL)
      {
        p_parser->pf_on_song_select(p_parser->p_cb_data,
                                    douze_GetByte(p_message));
      }
      douze_ReleaseMessage(p_demux, p_message);
      break;

    case 0xf5:
      /* Bus Select (Unofficial) */
      if(p_parser->pf_on_bus_select != NULL)
      {
        p_parser->pf_on_bus_select(p_parser->p_cb_data,
                                   douze_GetByte(p_message));
      }
      douze_ReleaseMessage(p_demux, p_message);
      break;

    case 0xf6:
      /* Tune Request */
      if(p_parser->pf_on_tune_request != NULL)
        p_parser->pf_on_tune_request(p_parser->p_cb_data);
      douze_ReleaseMessage(p_demux, p_message);
      break;

    case 0xf8:
      /* Midi Clock */
      if(p_parser->pf_on_midi_clock != NULL)
        p_parser->pf_on_midi_clock(p_parser->p_cb_data);
      douze_ReleaseMessage(p_demux, p_message);
      break;

    case 0xf9:
      /* Tick */
      if(p_parser->pf_on_tick != NULL)
        p_parser->pf_on_tick(p_parser->p_cb_data);
      douze_ReleaseMessage(p_demux, p_message);
      break;

    case 0xfa:
      /* Midi Start */
      if(p_parser->pf_on_midi_start != NULL)
        p_parser->pf_on_midi_start(p_parser->p_cb_data);
      douze_ReleaseMessage(p_demux, p_message);
      break;

    case 0xfb:
      /* Midi Continue */
      if(p_parser->pf_on_midi_continue != NULL)
        p_parser->pf_on_midi_continue(p_parser->p_cb_data);
      douze_ReleaseMessage(p_demux, p_message);
      break;

    case 0xfc:
      /* Midi Stop */
      if(p_parser->pf_on_midi_stop != NULL)
        p_parser->pf_on_midi_stop(p_parser->p_cb_data);
      douze_ReleaseMessage(p_demux, p_message);
      break;

    case 0xfe:
      /* Active Sense */
      if(p_parser->pf_on_active_sense != NULL)
        p_parser->pf_on_active_sense(p_parser->p_cb_data);
      douze_ReleaseMessage(p_demux, p_message);
      break;

    case 0xff:
      /* Reset */
      if(p_parser->pf_on_reset != NULL)
        p_parser->pf_on_reset(p_parser->p_cb_data);
      douze_ReleaseMessage(p_demux, p_message);
      break;
    }
    break;

  default:
    /* Never happens */
    fprintf(stderr, DOUZE_PARSER "malformed message received, trashed\n");
    douze_ReleaseMessage(p_demux, p_message);
    break;
  }
}

