/*****************************************************************************
 * livedrive_decoder.c - Live! Drive IR Remote control decoder
 *****************************************************************************
 * Copyright (C) 2001 Arnaud de Bossoreille de Ribou
 *
 * Authors: Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/


/*****************************************************************************
 * Description :
 *
 * implementation of a Live! Drive IR decoder.
 *****************************************************************************/


#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>

#include "common.h"
#include "douze.h"
#include "douze_parser.h"
#include "livedrive_decoder.h"


/*****************************************************************************
 * s_creative_devs
 *****************************************************************************
 * global list of devices
 *****************************************************************************/
static creative_dev_desc_t s_creative_devs[] =
{
  { CL_DEVID_RM900,             "RM-900"                },
  { CL_DEVID_DTT3500,           "DTT3500"               },

  /* End of the list */
  { CL_DEVID_NULL,              "Unknown device"        }
};


/*****************************************************************************
 * s_creative_keys
 *****************************************************************************
 * global list of keys
 *****************************************************************************/
static creative_key_desc_t s_creative_keys[] =
{
  /* Live! Drive IR RM-900 (Creative) */
  { CL_KEYID_RM900_1,                   CL_DEVID_RM900,     "1"             },
  { CL_KEYID_RM900_2,                   CL_DEVID_RM900,     "2"             },
  { CL_KEYID_RM900_3,                   CL_DEVID_RM900,     "3"             },
  { CL_KEYID_RM900_4,                   CL_DEVID_RM900,     "4"             },
  { CL_KEYID_RM900_5,                   CL_DEVID_RM900,     "5"             },
  { CL_KEYID_RM900_6,                   CL_DEVID_RM900,     "6"             },
  { CL_KEYID_RM900_7,                   CL_DEVID_RM900,     "7"             },
  { CL_KEYID_RM900_8,                   CL_DEVID_RM900,     "8"             },
  { CL_KEYID_RM900_9,                   CL_DEVID_RM900,     "9"             },
  { CL_KEYID_RM900_0,                   CL_DEVID_RM900,     "0"             },

  { CL_KEYID_RM900_STOP,                CL_DEVID_RM900,     "Stop"          },
  { CL_KEYID_RM900_PLAYPAUSE,           CL_DEVID_RM900,     "Play/Pause"    },
  { CL_KEYID_RM900_PREV,                CL_DEVID_RM900,     "Previous"      },
  { CL_KEYID_RM900_NEXT,                CL_DEVID_RM900,     "Next"          },
  { CL_KEYID_RM900_SLOW,                CL_DEVID_RM900,     "Slow"          },
  { CL_KEYID_RM900_STEP,                CL_DEVID_RM900,     "Step"          },

  { CL_KEYID_RM900_MUTE,                CL_DEVID_RM900,     "Mute"          },
  { CL_KEYID_RM900_VOLMINUS,            CL_DEVID_RM900,     "Volume -"      },
  { CL_KEYID_RM900_VOLPLUS,             CL_DEVID_RM900,     "Volume +"      },

  { CL_KEYID_RM900_EAX,                 CL_DEVID_RM900,     "EAX"           },
  { CL_KEYID_RM900_OPTIONS,             CL_DEVID_RM900,     "Options"       },
  { CL_KEYID_RM900_DISPLAY,             CL_DEVID_RM900,     "Display"       },
  { CL_KEYID_RM900_RETURN,              CL_DEVID_RM900,     "Return"        },
  { CL_KEYID_RM900_START,               CL_DEVID_RM900,     "Start"         },
  { CL_KEYID_RM900_CLOSE,               CL_DEVID_RM900,     "Close"         },

  { CL_KEYID_RM900_UP,                  CL_DEVID_RM900,     "Up"            },
  { CL_KEYID_RM900_DOWN,                CL_DEVID_RM900,     "Down"          },
  { CL_KEYID_RM900_LEFT,                CL_DEVID_RM900,     "Left"          },
  { CL_KEYID_RM900_RIGHT,               CL_DEVID_RM900,     "Right"         },
  { CL_KEYID_RM900_SELECT,              CL_DEVID_RM900,     "Select"        },

  /* Cambridge SoundWorks DTT3500 (Creative) */
  { CL_KEYID_DTT3500_POWER,             CL_DEVID_DTT3500,   "Power"         },

  { CL_KEYID_DTT3500_VOLPLUS,           CL_DEVID_DTT3500,   "Volume +"      },
  { CL_KEYID_DTT3500_VOLMINUS,          CL_DEVID_DTT3500,   "Volume -"      },

  { CL_KEYID_DTT3500_MUTE,              CL_DEVID_DTT3500,   "Mute"          },
  { CL_KEYID_DTT3500_TEST,              CL_DEVID_DTT3500,   "Test"          },

  { CL_KEYID_DTT3500_CMSS,              CL_DEVID_DTT3500,   "CMSS"          },
  { CL_KEYID_DTT3500_MULTICHANNEL,      CL_DEVID_DTT3500,   "Multi Channel" },
  { CL_KEYID_DTT3500_PROLOGIC,          CL_DEVID_DTT3500,   "ProLogic"      },
  { CL_KEYID_DTT3500_DYNAMICMODE,       CL_DEVID_DTT3500,   "Dynamic mode"  },
  { CL_KEYID_DTT3500_DOLBYDIGITAL,      CL_DEVID_DTT3500,   "Dolby Digital" },
  { CL_KEYID_DTT3500_AUDIOMODE,         CL_DEVID_DTT3500,   "Audio mode"    },

  /* End of the list */
  { CL_KEYID_NULL,                      CL_DEVID_NULL,      "Unknown key"   }
};


/*****************************************************************************
 * creative_FindDevDesc
 *****************************************************************************
 * Find the structure associated to a device id.
 *****************************************************************************/
creative_dev_desc_t * creative_FindDevDesc(u32 ui_dev_id)
{
  unsigned int ui = 0;

  while(    (s_creative_devs[ui].ui_dev_id != ui_dev_id)
         && (s_creative_devs[ui].ui_dev_id != CL_DEVID_NULL))
    ui++;

  return &s_creative_devs[ui];
}


/*****************************************************************************
 * creative_FindKeyDesc
 *****************************************************************************
 * Find the structure associated to a key id.
 *****************************************************************************/
creative_key_desc_t * creative_FindKeyDesc(u32 ui_key_id)
{
  unsigned int ui = 0;

  while(    (s_creative_keys[ui].ui_key_id != ui_key_id)
         && (s_creative_keys[ui].ui_key_id != CL_KEYID_NULL))
    ui++;

  return &s_creative_keys[ui];
}


/*****************************************************************************
 * livedrive_InitDecoder
 *****************************************************************************
 * This function initializes a livedrive_decoder_t structure.
 *****************************************************************************/
void livedrive_InitDecoder(livedrive_decoder_t *        p_decoder,
                           livedrive_HandleKeyFunc      pf_handle_key,
                           void *                       p_cb_data)
{
  p_decoder->pf_handle_key = pf_handle_key;
  p_decoder->p_cb_data = p_cb_data;
}

/*****************************************************************************
 * livedrive_OnSystemExclusive
 *****************************************************************************
 * parser callback functions
 *****************************************************************************/
void livedrive_OnSystemExclusive(livedrive_decoder_t * p_decoder,
                                 u16 ui_manufacturer_id,
                                 douze_sysex_message_t * p_sysex)
{
  if(ui_manufacturer_id == (0x8000 | 0x2021))
  {
    if(    (p_sysex->ui_payload_size == 8)
        && (douze_GetByte(p_sysex->p_message) == 0x60))
    {
      u32 ui_key_id;
      u32 data;

      douze_GetByte(p_sysex->p_message);
      douze_GetByte(p_sysex->p_message);
      data = douze_GetByte(p_sysex->p_message);
      ui_key_id =   ((data & 0x01) << 31)
                  | ((data & 0x02) << 22)
                  | ((data & 0x04) << 13)
                  | ((data & 0x08) << 4);
      ui_key_id |= douze_GetByte(p_sysex->p_message);
      ui_key_id |= douze_GetByte(p_sysex->p_message) << 8;
      ui_key_id |= douze_GetByte(p_sysex->p_message) << 16;
      ui_key_id |= douze_GetByte(p_sysex->p_message) << 24;

      p_decoder->pf_handle_key(p_decoder->p_cb_data, ui_key_id);

    }
  }

  douze_ReleaseSysExMessage(p_sysex);
}

