/*****************************************************************************
 * midi_decode.c - Midi decoder program
 *****************************************************************************
 * Copyright (C) 2001 Arnaud de Bossoreille de Ribou
 *
 * Authors: Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/


/*****************************************************************************
 * Description :
 *
 * this program decodes all valid midi messages received by the demultiplexer.
 *****************************************************************************/


#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>

#include "common.h"
#include "douze.h"
#include "douze_parser.h"


/*****************************************************************************
 * logging string
 *****************************************************************************/
#define DOUZE_DECODER       "midi decoder : "


/*****************************************************************************
 * netlist size (10000 packets of 3 bytes => 30ko)
 *****************************************************************************/
#define DD_PACKET_SIZE      3
#define DD_LIFO_SIZE        10000


/*****************************************************************************
 * main_thread_t
 *****************************************************************************
 * This structure is used to store global data of the program.
 *****************************************************************************/
typedef struct main_thread_s
{
  douze_demux_t         demux;
  douze_parser_t        parser;
  boolean_t             b_stop;
  int                   i_fd;
  byte_t                buffer[DD_PACKET_SIZE * DD_LIFO_SIZE];
  douze_packet_t        p_packets[DD_LIFO_SIZE];
  douze_packet_t *      lifo[DD_LIFO_SIZE];
  unsigned int          ui_next;

} main_thread_t;


/*****************************************************************************
 * main structure.
 *****************************************************************************/
static main_thread_t * s_p_main = NULL;


/*****************************************************************************
 * parser callback functions
 *****************************************************************************/
void OnNoteOff(void * p_data,
               u8 ui_channel, u8 ui_note, u8 ui_velocity)
{
  fprintf(stdout,
          "Midi Note Off ( channel == %u, note == %u, velocity == %u )\n",
          (unsigned int)ui_channel,
          (unsigned int)ui_note,
          (unsigned int)ui_velocity);
}


/*****************************************************************************/
void OnNoteOn(void * p_data,
              u8 ui_channel, u8 ui_note, u8 ui_velocity)
{
  fprintf(stdout,
          "Midi Note On ( channel == %u, note == %u, velocity == %u )\n",
          (unsigned int)ui_channel,
          (unsigned int)ui_note,
          (unsigned int)ui_velocity);
}


/*****************************************************************************/
void OnAftertouch(void * p_data,
                  u8 ui_channel, u8 ui_note, u8 ui_pressure)
{
  fprintf(stdout,
          "Midi Aftertouch ( channel == %u, note == %u, pressure == %u )\n",
          (unsigned int)ui_channel,
          (unsigned int)ui_note,
          (unsigned int)ui_pressure);
}


/*****************************************************************************/
void OnController(void * p_data,
                  u8 ui_channel, u8 ui_controller, u8 ui_value)
{
  fprintf(stdout,
          "Midi Controller ( channel == %u, controller == %u, value == %u )\n",
          (unsigned int)ui_channel,
          (unsigned int)ui_controller,
          (unsigned int)ui_value);
}


/*****************************************************************************/
void OnProgramChange(void * p_data,
                     u8 ui_channel, u8 ui_program)
{
  fprintf(stdout, "Midi Program Change ( channel == %u, program == %u )\n",
          (unsigned int)ui_channel,
          (unsigned int)ui_program);
}


/*****************************************************************************/
void OnChannelPressure(void * p_data,
                       u8 ui_channel, u8 ui_pressure)
{
  fprintf(stdout, "Midi Channel Pressure ( channel == %u, pressure == %u )\n",
          (unsigned int)ui_channel,
          (unsigned int)ui_pressure);
}


/*****************************************************************************/
void OnPitchWheel(void * p_data,
                  u8 ui_channel, u16 ui_value)
{
  fprintf(stdout, "Midi Pitch Wheel ( channel == %u, value == %u )\n",
          (unsigned int)ui_channel,
          (unsigned int)ui_value);
}


/*****************************************************************************/
void OnSystemExclusive(void * p_data,
                       u16 ui_manufacturer_id,
                       douze_sysex_message_t * p_sysex)
{
  unsigned int ui;

  fprintf(stdout, "Midi System Exclusive ( ");

  if(ui_manufacturer_id & 0x8000)
  {
    fprintf(stdout, "extended manufacturer id == 0x%04x",
            ui_manufacturer_id & ~0x8000);
  }
  else
  {
    fprintf(stdout, "classic manufacturer id == 0x%02x", ui_manufacturer_id);
  }

  fprintf(stdout, ", payload == | ");
  for(ui = 0; ui < p_sysex->ui_payload_size; ui++)
    fprintf(stdout, "%02x ", douze_GetByte(p_sysex->p_message));
  fprintf(stdout, "| )\n");

  douze_ReleaseSysExMessage(p_sysex);
}


/*****************************************************************************/
void OnMTCQuarterFrameMsg(void * p_data,
                          u8 ui_time_code)
{
  fprintf(stdout, "Midi MTC Quarter Frame Message ( time code == %u )\n",
          (unsigned int)ui_time_code);
}


/*****************************************************************************/
void OnSongPositionPointer(void * p_data,
                           u16 ui_beat)
{
  fprintf(stdout,
          "Midi Song Position Pointer ( beat == %u )\n",
          (unsigned int)ui_beat);
}


/*****************************************************************************/
void OnSongSelect(void * p_data,
                  u8 ui_song)
{
  fprintf(stdout, "Midi Song Select ( song == %u )\n", (unsigned int)ui_song);
}


/*****************************************************************************/
void OnBusSelect(void * p_data,
                 u8 ui_bus)
{
  fprintf(stdout, "Midi Bus Select ( song == %u )\n", (unsigned int)ui_bus);
}


/*****************************************************************************/
void OnTuneRequest(void * p_data)
{
  fprintf(stdout, "Midi Tune Request ( )\n");
}


/*****************************************************************************/
void OnMidiClock(void * p_data)
{
  fprintf(stdout, "Midi Midi Clock ( )\n");
}


/*****************************************************************************/
void OnTick(void * p_data)
{
  fprintf(stdout, "Midi Tick ( )\n");
}


/*****************************************************************************/
void OnMidiStart(void * p_data)
{
  fprintf(stdout, "Midi Midi Start ( )\n");
}


/*****************************************************************************/
void OnMidiContinue(void * p_data)
{
  fprintf(stdout, "Midi Midi Continue ( )\n");
}


/*****************************************************************************/
void OnMidiStop(void * p_data)
{
  fprintf(stdout, "Midi Midi Stop ( )\n");
}


/*****************************************************************************/
void OnActiveSense(void * p_data)
{
  fprintf(stdout, "Midi Active Sense ( )\n");
}


/*****************************************************************************/
void OnReset(void * p_data)
{
  fprintf(stdout, "Midi Reset ( )\n");
}


/*****************************************************************************
 * GetPacket
 *****************************************************************************
 * This function gats a free packet in the netlist.
 *****************************************************************************/
douze_packet_t * GetPacket(main_thread_t * p_main)
{
  if(p_main->ui_next < DD_LIFO_SIZE)
  {
    douze_packet_t * p_packet = p_main->lifo[p_main->ui_next++];
    return p_packet;
  }
  else
    return NULL;
}


/*****************************************************************************
 * ReleasePacket
 *****************************************************************************
 * This function queues the released packet into the netlist.
 *****************************************************************************/
void ReleasePacket(main_thread_t * p_main, douze_packet_t * p_packet)
{
  p_main->lifo[--p_main->ui_next] = p_packet;
}


/*****************************************************************************
 * SignalHandler
 *****************************************************************************/
void SignalHandler(int i_signal)
{
  switch(i_signal)
  {
  case SIGINT:
    fprintf(stderr, DOUZE_DECODER "SIGINT signal received\n");
    if(s_p_main)
      s_p_main->b_stop = 1;
    break;
  default:
    fprintf(stderr, DOUZE_DECODER "unhandled signal received (%d)\n", i_signal);
    break;
  }
}


/*****************************************************************************
 * InstallSignalHandlers
 *****************************************************************************
 * This function installs a handler for system signals.
 *****************************************************************************/
int InstallSignalHandlers()
{
  sigset_t sSigSet;
  struct sigaction sSigAction;

  sigemptyset(&sSigSet);
  sSigAction.sa_mask = sSigSet;
  sSigAction.sa_flags = 0;

  sSigAction.sa_handler = SignalHandler;
  if(sigaction(SIGINT, &sSigAction, NULL) != 0)
  {
    fprintf(stderr, DOUZE_DECODER "unable to install handler for SIGINT\n");
    return -1;
  }
  return 0;
}


/*****************************************************************************
 * main function
 *****************************************************************************/
int main(int i_argc, char *ppsz_argv[], char *ppsz_env[])
{
  int i_rc = 0;
  main_thread_t main_thread;
  main_thread.b_stop = 0;

  fprintf(stdout,
    "Midi messages decoder - (C) 2001 Arnaud de Bossoreille de Ribou\n\n");

  if((i_argc != 2) || !strcmp(ppsz_argv[1], "--help"))
  {
    fprintf(stdout,
      "Usage: midi_decode device\n"
      "\n"
      "Arguments:\n"
      "  device: midi device where to read midi data (- == stdin)\n");
    return 0;;
  }

  /* Set static data pointer */
  s_p_main = &main_thread;

  /* Install signal handlers */
  if(!i_rc)
    i_rc = InstallSignalHandlers();

  /* Open the midi device */
  if(!i_rc)
  {
    if(strcmp(ppsz_argv[1], "-"))
    {
      main_thread.i_fd = open(ppsz_argv[1], O_RDONLY);
      i_rc = (main_thread.i_fd == -1) ? -1 : 0;
    }
    else
      main_thread.i_fd = 0; /* open stdin */

    if(i_rc)
      fprintf(stderr, DOUZE_DECODER "unable to open %s\n", ppsz_argv[1]);
  }

  if(!i_rc)
  {
    unsigned int ui;

    /* Initialize the main structure (lifo...) */
    for(ui = 0; ui < DD_LIFO_SIZE; ui++)
    {
      main_thread.p_packets[ui].p_buffer_start =
                                &main_thread.buffer[DD_PACKET_SIZE * ui];
      main_thread.p_packets[ui].p_buffer_end   =
                                &main_thread.buffer[DD_PACKET_SIZE * (ui + 1)];
      main_thread.lifo[ui] = &main_thread.p_packets[ui];
    }
    main_thread.ui_next = 0;

    /* Initialize the midi demultiplexer */
    douze_InitDemux(&main_thread.demux,
                    (douze_GetPacketFunc)GetPacket,
                    (douze_ReleasePacketFunc)ReleasePacket,
                    (douze_ParseMessageFunc)douze_ParseMessage,
                    &main_thread, &main_thread.parser);

    /* Initialize the midi parser */
    douze_InitParser(&main_thread.parser,
                     (douze_OnNoteOffFunc)OnNoteOff,
                     (douze_OnNoteOnFunc)OnNoteOn,
                     (douze_OnAftertouchFunc)OnAftertouch,
                     (douze_OnControllerFunc)OnController,
                     (douze_OnProgramChangeFunc)OnProgramChange,
                     (douze_OnChannelPressureFunc)OnChannelPressure,
                     (douze_OnPitchWheelFunc)OnPitchWheel,
                     (douze_OnSystemExclusiveFunc)OnSystemExclusive,
                     (douze_OnMTCQuarterFrameMsgFunc)OnMTCQuarterFrameMsg,
                     (douze_OnSongPositionPointerFunc)OnSongPositionPointer,
                     (douze_OnSongSelectFunc)OnSongSelect,
                     (douze_OnBusSelectFunc)OnBusSelect,
                     (douze_OnTuneRequestFunc)OnTuneRequest,
                     (douze_OnMidiClockFunc)OnMidiClock,
                     (douze_OnTickFunc)OnTick,
                     (douze_OnMidiStartFunc)OnMidiStart,
                     (douze_OnMidiContinueFunc)OnMidiContinue,
                     (douze_OnMidiStopFunc)OnMidiStop,
                     (douze_OnActiveSenseFunc)OnActiveSense,
                     (douze_OnResetFunc)OnReset,
                     NULL);

    /* main loop */
    while(!main_thread.b_stop)
    {
      byte_t data;
      if(read(main_thread.i_fd, &data, 1) == 1)
        douze_DemuxByte(&main_thread.demux, data);
      else
        main_thread.b_stop = 1;
    }
  }

  /* Close the midi device */
  if(main_thread.i_fd >= 0)
    i_rc |= close(main_thread.i_fd);

  /* Release the demultiplexer */
  douze_ReleaseDemux(&main_thread.demux);

  fprintf(stdout, DOUZE_DECODER "exiting ...\n");

  return i_rc;
}

