/*****************************************************************************
 * midi_dump.c - Midi dumper program
 *****************************************************************************
 * Copyright (C) 2001 Arnaud de Bossoreille de Ribou
 *
 * Authors: Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/


/*****************************************************************************
 * Description :
 *
 * this program dumps all valid midi messages received by the demultiplexer.
 *****************************************************************************/


#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>

#include "common.h"
#include "douze.h"


/*****************************************************************************
 * logging string
 *****************************************************************************/
#define DOUZE_DUMPER        "midi dumper : "


/*****************************************************************************
 * netlist size (10000 packets of 3 bytes => 30ko)
 *****************************************************************************/
#define DD_PACKET_SIZE      3
#define DD_LIFO_SIZE        10000


/*****************************************************************************
 * main_thread_t
 *****************************************************************************
 * This structure is used to store global data of the program.
 *****************************************************************************/
typedef struct main_thread_s
{
  douze_demux_t         demux;
  boolean_t             b_stop;
  int                   i_fd;
  byte_t                buffer[DD_PACKET_SIZE * DD_LIFO_SIZE];
  douze_packet_t        p_packets[DD_LIFO_SIZE];
  douze_packet_t *      lifo[DD_LIFO_SIZE];
  unsigned int          ui_next;

} main_thread_t;


/*****************************************************************************
 * main structure.
 *****************************************************************************/
static main_thread_t * s_p_main = NULL;


/*****************************************************************************
 * ParseMessage
 *****************************************************************************
 * This function dumps the midi message to stdout.
 *****************************************************************************/
void ParseMessage(void * p_data,
                  douze_demux_t * p_demux, douze_message_t * p_message)
{
  unsigned int ui;

  /* Dump the midi message */
  fprintf(stdout, "Midi message    | ");
  for(ui = 0; ui < p_message->ui_message_size; ui++)
    fprintf(stdout, "%02x ", douze_GetByte(p_message));
  fprintf(stdout, "|\n");

  /* Release the midi message */
  douze_ReleaseMessage(p_demux, p_message);
}


/*****************************************************************************
 * GetPacket
 *****************************************************************************
 * This function gats a free packet in the netlist.
 *****************************************************************************/
douze_packet_t * GetPacket(main_thread_t * p_main)
{
  if(p_main->ui_next < DD_LIFO_SIZE)
  {
    douze_packet_t * p_packet = p_main->lifo[p_main->ui_next++];
    return p_packet;
  }
  else
    return NULL;
}


/*****************************************************************************
 * ReleasePacket
 *****************************************************************************
 * This function queues the released packet into the netlist.
 *****************************************************************************/
void ReleasePacket(main_thread_t * p_main, douze_packet_t * p_packet)
{
  p_main->lifo[--p_main->ui_next] = p_packet;
}


/*****************************************************************************
 * SignalHandler
 *****************************************************************************/
void SignalHandler(int i_signal)
{
  switch(i_signal)
  {
  case SIGINT:
    fprintf(stderr, DOUZE_DUMPER "SIGINT signal received\n");
    if(s_p_main)
      s_p_main->b_stop = 1;
    break;
  default:
    fprintf(stderr, DOUZE_DUMPER "unhandled signal received (%d)\n", i_signal);
    break;
  }
}


/*****************************************************************************
 * InstallSignalHandlers
 *****************************************************************************
 * This function installs a handler for system signals.
 *****************************************************************************/
int InstallSignalHandlers()
{
  sigset_t sSigSet;
  struct sigaction sSigAction;

  sigemptyset(&sSigSet);
  sSigAction.sa_mask = sSigSet;
  sSigAction.sa_flags = 0;

  sSigAction.sa_handler = SignalHandler;
  if(sigaction(SIGINT, &sSigAction, NULL) != 0)
  {
    fprintf(stderr, DOUZE_DUMPER "unable to install handler for SIGINT\n");
    return -1;
  }
  return 0;
}


/*****************************************************************************
 * main function
 *****************************************************************************/
int main(int i_argc, char *ppsz_argv[], char *ppsz_env[])
{
  int i_rc = 0;
  main_thread_t main_thread;
  main_thread.b_stop = 0;

  fprintf(stdout,
    "Midi messages dumper - (C) 2001 Arnaud de Bossoreille de Ribou\n\n");

  if((i_argc != 2) || !strcmp(ppsz_argv[1], "--help"))
  {
    fprintf(stdout,
      "Usage: midi_dump device\n"
      "\n"
      "Arguments:\n"
      "  device: midi device where to read midi data (- == stdin)\n");
    return 0;;
  }

  /* Set static data pointer */
  s_p_main = &main_thread;

  /* Install signal handlers */
  if(!i_rc)
    i_rc = InstallSignalHandlers();

  /* Open the midi device */
  if(!i_rc)
  {
    if(strcmp(ppsz_argv[1], "-"))
    {
      main_thread.i_fd = open(ppsz_argv[1], O_RDONLY);
      i_rc = (main_thread.i_fd == -1) ? -1 : 0;
    }
    else
      main_thread.i_fd = 0; /* open stdin */

    if(i_rc)
      fprintf(stderr, DOUZE_DUMPER "unable to open %s\n", ppsz_argv[1]);
  }

  if(!i_rc)
  {
    unsigned int ui;

    /* Initialize the main structure (lifo...) */
    for(ui = 0; ui < DD_LIFO_SIZE; ui++)
    {
      main_thread.p_packets[ui].p_buffer_start =
                                &main_thread.buffer[DD_PACKET_SIZE * ui];
      main_thread.p_packets[ui].p_buffer_end   =
                                &main_thread.buffer[DD_PACKET_SIZE * (ui + 1)];
      main_thread.lifo[ui] = &main_thread.p_packets[ui];
    }
    main_thread.ui_next = 0;

    /* Initialize the midi demultiplexer */
    douze_InitDemux(&main_thread.demux,
                    (douze_GetPacketFunc)GetPacket,
                    (douze_ReleasePacketFunc)ReleasePacket,
                    (douze_ParseMessageFunc)ParseMessage,
                    &main_thread, NULL);

    /* main loop */
    while(!main_thread.b_stop)
    {
      byte_t data;
      if(read(main_thread.i_fd, &data, 1) == 1)
        douze_DemuxByte(&main_thread.demux, data);
      else
        main_thread.b_stop = 1;
    }
  }

  /* Close the midi device */
  if(main_thread.i_fd >= 0)
    i_rc |= close(main_thread.i_fd);

  /* Release the demultiplexer */
  douze_ReleaseDemux(&main_thread.demux);

  fprintf(stdout, DOUZE_DUMPER "exiting ...\n");

  return i_rc;
}

